import { 
  select,
  json,
  scaleOrdinal,
  range,
  schemeSpectral,
  interpolateViridis
} from 'd3';
import { colorLegend } from './colorLegend';
import { scatterPlot } from './scatterPlot';

const svg = select('svg').style('background-color', '#f7ef9f');
const width  = +svg.attr('width');
const height = +svg.attr('height');

const midiFileColorLegendG = svg.append('g')
    .attr('transform', `translate(50,40)`);
const colorScale = scaleOrdinal();

const trackColorLegendG = svg.append('g')
    .attr('transform', `translate(50,200)`);
const trackColorScale = scaleOrdinal();

const scatterPlotG = svg.append('g');

const notesColorScale = scaleOrdinal();
const colorValue = d => d.numero;

let midiFiles;
let filesAndTracks;
let selectedFile;
let selectedTrack;
let tickLength;

const midiFileOnClick = d => {
  selectedFile = d;
  const tracks = getTracks();
  selectedTrack = tracks[0];
  render();
};

const trackOnClick = d => {
  selectedTrack = d;
  render();
  //renderTracks();
};

const getTracks = () => {
  return Object.keys(
    midiFiles[selectedFile].notas
  );
}

const getFileTrack = () => {
  const mf = midiFiles[selectedFile];
  tickLength = +mf.tickLength;
  const dd = mf.notas[selectedTrack];
  dd.forEach(d => {
    d.numero = +d.numero;
    d.inicio = +d.inicio;
    d.fim = +d.fim;
    d.duracao = +d.duracao;
    d.channel = +d.channel;
  });
  return dd;
};

const render = () => {
  renderMidifiles();
  renderTracks();
  renderScatterPlot();
};

const renderMidifiles = () => {
  colorScale
    .domain(range(filesAndTracks.length))
    .range(schemeSpectral[colorScale.domain().length]);
  
  midiFileColorLegendG.call(colorLegend, {
    colorScale,
    circleRadius: 8,
    spacing: 25,
    textOffset: 15,
    backgroundRectWidth: 65,
    onClick: midiFileOnClick,
    selectedColorValue: selectedFile
  });
}

const renderTracks = () => {
  const tracks = getTracks();
  const nn = tracks.length;
  const trackColorScale = scaleOrdinal()
    .domain(tracks)
    .range([0, 255]);
  
  trackColorLegendG.call(colorLegend, {
    colorScale: trackColorScale,
    circleRadius: 8,
    spacing: 25,
    textOffset: 12,
    backgroundRectWidth: 65,
    onClick: trackOnClick,
    selectedColorValue: selectedTrack
  });
}

const renderScatterPlot = () => {

  const dd = getFileTrack();
  
  notesColorScale
    .domain(dd.map(colorValue))
    .domain(notesColorScale.domain().sort());
  
  console.log(dd.map(colorValue))

    console.log(notesColorScale.domain());
  console.log(notesColorScale.range());

  
  const n = notesColorScale.domain().length;
  
  notesColorScale.range(notesColorScale.domain().map((d, i) =>
    interpolateViridis(i / (n + 1))
  ));
  console.log(notesColorScale.range());
  
  scatterPlotG.call(scatterPlot, {
    data: dd,
    width,
    height,
    margin: { top: 50, right: 40, bottom: 88, left: 180 },
    title: 'Notes: start, duration & end',
    xValueStart: d => d.inicio,
    xValueEnd: d => d.fim,
    xValueDur: d => d.duracao,
    xAxisLabel: 'Ticks',
    yValue: d => d.numero,
    yAxisLabel: 'Notes',
    maxValue: tickLength,
    notesColorScale,
    colorValue
  });

};

json('https://vizhub.com/rubens2005/datasets/songs.csv').then(data => {
  midiFiles = data;
  
  const keys = Object.keys(data);
  filesAndTracks = keys.map(d => Object.keys(data[d].notas));
  selectedFile = 2;
  selectedTrack = filesAndTracks[selectedFile][0];
  
  render();
});