import { 
  select, 
  scaleLinear, 
  axisLeft,
  axisBottom,
  extent
} from 'd3';

export const scatterPlot = (selection, props) => {
  const {
    data,
    width,
    height,
    margin,
    title,
    xValueStart,
    xValueEnd,
    xValueDur,
    xAxisLabel,
    yValue,
    yAxisLabel,
    maxValue,
    notesColorScale,
    colorValue
  } = props;
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  const xScale = scaleLinear()
    .domain([0, maxValue])
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
    .enter().append('g')
      .attr('class', 'container');
  gEnter
    .merge(g)
      .attr('transform',
        `translate(${margin.left},${margin.top})`
      );
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
      .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
      .call(yAxis)
      .selectAll('.domain').remove();
  
  const yAxisLabelText = yAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', -50)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
    .merge(yAxisG.select('.axis-label'))
      .attr('x', -innerHeight / 2)
      .text(yAxisLabel);
  
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
      .attr('class', 'x-axis');
  xAxisG
    .merge(xAxisGEnter)
      .attr('transform', `translate(0,${innerHeight})`)
      .call(xAxis)
      .selectAll('.domain').remove();
  
  const xAxisLabelText = xAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);

  console.log(notesColorScale.domain());
  console.log(notesColorScale.range());
  
  
  const rects = g.merge(gEnter)
    .selectAll('rect').data(data);
		rects.enter().append('rect')
    .merge(rects)
    .transition().duration(1000)
    .delay((d, i) => i * 0.75 )
      .attr('x', d => xScale(xValueStart(d)))
      .attr('y', d => yScale(yValue(d)) - 6)
      .attr('width', d => xScale(xValueDur(d)))
      .attr('height', 10)
      .style('fill', d => {
          console.log(notesColorScale(colorValue(d)));
          return notesColorScale(colorValue(d));
                          });
  rects.exit().remove();
  
	gEnter.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
}